"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadJatsFromUrl = exports.checkIfPubMedCentralHasJats = exports.convertPMID2PMCID = void 0;
const doi_utils_1 = __importDefault(require("doi-utils"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const myst_cli_utils_1 = require("myst-cli-utils");
const resolvers_1 = require("../resolvers");
function downloadFromUrl(session, jatsUrl, opts) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const toc = (0, myst_cli_utils_1.tic)();
        session.log.debug(`Fetching JATS from ${jatsUrl}`);
        const resp = yield ((_a = opts === null || opts === void 0 ? void 0 : opts.fetcher) !== null && _a !== void 0 ? _a : defaultFetcher)(jatsUrl, 'xml');
        if (!resp.ok) {
            session.log.debug(`JATS failed to download from "${jatsUrl}"`);
            throw new Error(`STATUS ${resp.status}: ${resp.statusText}`);
        }
        const contentType = (_b = resp.headers) === null || _b === void 0 ? void 0 : _b.get('content-type');
        if (!((contentType === null || contentType === void 0 ? void 0 : contentType.includes('application/xml')) ||
            (contentType === null || contentType === void 0 ? void 0 : contentType.includes('text/xml')) ||
            (contentType === null || contentType === void 0 ? void 0 : contentType.includes('text/plain')))) {
            session.log.warn(`Expected content-type "application/xml" instead we got "${contentType}" for ${jatsUrl}`);
        }
        const data = yield resp.text();
        session.log.debug(toc(`Fetched document with content-type "${contentType}" in %s`));
        return data;
    });
}
function defaultFetcher(url, kind) {
    switch (kind) {
        case 'json':
            return (0, node_fetch_1.default)(url, { headers: [['Accept', 'application/json']] });
        case 'xml':
            return (0, node_fetch_1.default)(url, { headers: [['Accept', 'application/xml']] });
        default:
            return (0, node_fetch_1.default)(url);
    }
}
/**
 * There are 5.8M or so DOIs that have a full XML record:
 *
 * https://api.crossref.org/works?filter=full-text.type:application/xml,full-text.application:text-mining&facet=publisher-name:*&rows=0
 *
 * This function tries to find the correct URL for the record.
 */
function checkIfDoiHasJats(session, urlOrDoi, opts) {
    var _a, _b, _c, _d, _e;
    return __awaiter(this, void 0, void 0, function* () {
        if (!doi_utils_1.default.validate(urlOrDoi))
            return;
        const toc = (0, myst_cli_utils_1.tic)();
        const doiUrl = doi_utils_1.default.buildUrl(urlOrDoi);
        session.log.debug(`Attempting to resolving full XML from DOI ${doiUrl}`);
        const resp = yield ((_a = opts === null || opts === void 0 ? void 0 : opts.fetcher) !== null && _a !== void 0 ? _a : defaultFetcher)(doiUrl, 'json');
        if (!resp.ok) {
            // Silently return -- other functions can try!
            session.log.debug(`DOI failed to resolve: ${doiUrl}`);
            return;
        }
        const data = (yield resp.json());
        session.log.debug(toc(`DOI resolved in %s with ${(_c = (_b = data.link) === null || _b === void 0 ? void 0 : _b.length) !== null && _c !== void 0 ? _c : 0} links to content`));
        if (data.link) {
            session.log.debug(['', ...data.link.map((link) => `content-type: ${link['content-type']}, ${link.URL}\n`)].join('  - '));
        }
        const fullXml = (_e = (_d = data.link) === null || _d === void 0 ? void 0 : _d.find((link) => { var _a; return ['text/xml', 'application/xml'].includes((_a = link['content-type']) !== null && _a !== void 0 ? _a : ''); })) === null || _e === void 0 ? void 0 : _e.URL;
        if (fullXml)
            return fullXml;
        session.log.debug(`Could not find XML in DOI record ${doiUrl}`);
        return undefined;
    });
}
/**
 * https://www.ncbi.nlm.nih.gov/pmc/tools/id-converter-api/
 */
function convertPMID2PMCID(session, PMID, opts) {
    var _a, _b, _c;
    return __awaiter(this, void 0, void 0, function* () {
        if (PMID.startsWith('https://')) {
            const idPart = new URL(PMID).pathname.slice(1);
            session.log.debug(`Extract ${PMID} to ${idPart}`);
            return convertPMID2PMCID(session, idPart, opts);
        }
        const toc = (0, myst_cli_utils_1.tic)();
        const converter = 'https://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/';
        const resp = yield ((_a = opts === null || opts === void 0 ? void 0 : opts.fetcher) !== null && _a !== void 0 ? _a : defaultFetcher)(`${converter}?tool=jats-xml&format=json&ids=${PMID}`, 'json');
        if (!resp.ok) {
            // Silently return -- other functions can try!
            session.log.debug(`Failed to convert PubMedID: ${PMID}`);
            return;
        }
        const data = yield resp.json();
        const PMCID = (_c = (_b = data === null || data === void 0 ? void 0 : data.records) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.pmcid;
        session.log.debug(toc(`Used nih.gov to transform ${PMID} to ${PMCID} in %s.`));
        return PMCID;
    });
}
exports.convertPMID2PMCID = convertPMID2PMCID;
function pubMedCentralJats(PMCID) {
    const normalized = PMCID.replace(/^PMC:?/, '');
    return `https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pmc&id=${normalized}`;
}
function checkIfPubMedCentralHasJats(session, urlOrDoi, opts) {
    var _a, _b, _c;
    return __awaiter(this, void 0, void 0, function* () {
        if (urlOrDoi.match(/^PMC:?([0-9]+)$/))
            return pubMedCentralJats(urlOrDoi);
        if (!doi_utils_1.default.validate(urlOrDoi))
            return;
        const toc = (0, myst_cli_utils_1.tic)();
        const doiUrl = doi_utils_1.default.buildUrl(urlOrDoi);
        session.log.debug(`Attempting to resolve PMCID using OpenAlex from ${doiUrl}`);
        const openAlexUrl = `https://api.openalex.org/works/${doiUrl}`;
        const resp = yield ((_a = opts === null || opts === void 0 ? void 0 : opts.fetcher) !== null && _a !== void 0 ? _a : defaultFetcher)(openAlexUrl, 'json');
        if (!resp.ok) {
            // Silently return -- other functions can try!
            session.log.debug(`Failed to lookup on OpenAlex: ${openAlexUrl}`);
            return;
        }
        const data = (yield resp.json());
        const PMID = (_b = data === null || data === void 0 ? void 0 : data.ids) === null || _b === void 0 ? void 0 : _b.pmid;
        let PMCID = (_c = data === null || data === void 0 ? void 0 : data.ids) === null || _c === void 0 ? void 0 : _c.pmcid;
        if (!PMCID && !!PMID) {
            session.log.debug(toc(`OpenAlex resolved ${data === null || data === void 0 ? void 0 : data.ids.openalex} in %s. There is no PMCID, but there is a PMID`));
            PMCID = yield convertPMID2PMCID(session, PMID, opts);
            if (!PMCID) {
                session.log.debug(toc(`PubMed does not have a record of ${PMID}`));
                return;
            }
        }
        if (!PMCID) {
            session.log.debug(toc(`OpenAlex resolved ${data === null || data === void 0 ? void 0 : data.ids.openalex} in %s, but there is no PMCID`));
            return;
        }
        session.log.debug(toc(`OpenAlex resolved in %s, with a PMCID of ${PMCID}`));
        return pubMedCentralJats(PMCID);
    });
}
exports.checkIfPubMedCentralHasJats = checkIfPubMedCentralHasJats;
function downloadJatsFromUrl(session, urlOrDoi, opts = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const expectedUrls = (yield Promise.all([
            checkIfPubMedCentralHasJats(session, urlOrDoi, opts),
            checkIfDoiHasJats(session, urlOrDoi, opts),
        ])).filter((u) => !!u);
        if (expectedUrls.length > 0) {
            session.log.debug(['Trying URLs:\n', ...expectedUrls.map((url) => ` ${url}\n`)].join('  - '));
            for (let index = 0; index < expectedUrls.length; index++) {
                const url = expectedUrls[index];
                try {
                    const data = yield downloadFromUrl(session, url, opts);
                    if (data)
                        return { success: true, source: url, data };
                }
                catch (error) {
                    session.log.debug(error.message);
                }
            }
            // If there are expected URLs that don't work: see something, say something, etc.
            return { success: false, source: expectedUrls[0] };
        }
        if (doi_utils_1.default.validate(urlOrDoi)) {
            const jatsUrl = yield (0, resolvers_1.customResolveJatsUrlFromDoi)(session, urlOrDoi, opts);
            const data = yield downloadFromUrl(session, jatsUrl, opts);
            return { success: true, source: jatsUrl, data };
        }
        if ((0, myst_cli_utils_1.isUrl)(urlOrDoi)) {
            session.log.debug("No resolver matched, and the URL doesn't look like a DOI. We will attempt to download it directly.");
            const data = yield downloadFromUrl(session, urlOrDoi, opts);
            return { success: true, source: urlOrDoi, data };
        }
        throw new Error(`Could not find ${urlOrDoi} locally, and it doesn't look like a URL or DOI`);
    });
}
exports.downloadJatsFromUrl = downloadJatsFromUrl;
